/* eslint-disable no-magic-numbers */

import { findEncoding } from './solver.js';

import encodingImage from './images/encoding.svg';
import styles from './solution.module.css';

function formatMap(solution) {
  const contents = [...solution.entries()].map(([meaning, code]) => `${meaning} → ${code}`).join(', ');
  return `{${contents}}`;
}

function score(frequencies, encoding) {
  let result = 0;
  for (const [meaning, bits] of encoding) {
    result += frequencies.get(meaning) * bits.length;
  }
  return result;
}

const FIRST_PROBLEM_FREQUENCIES = new Map([
  ['a', 0.15],
  ['b', 0.30],
  ['c', 0.25],
  ['d', 0.30],
]);
const SECOND_PROBLEM_FREQUENCIES = new Map([
  ['a', 0.15],
  ['b', 0.40],
  ['c', 0.25],
  ['d', 0.20],
]);

export function Solution() {
  const firstSolution = findEncoding(FIRST_PROBLEM_FREQUENCIES);
  const secondSolution = findEncoding(SECOND_PROBLEM_FREQUENCIES);
  return (
    <div>
      <figure className={styles.diagram} >
        <img src={encodingImage} alt="A weighted directed graph corresponding to the encoding problem." />
      </figure>
      <label>
        Frequencies: {formatMap(FIRST_PROBLEM_FREQUENCIES)}<br/>
        Chosen Encoding:
        <output>
          {formatMap(firstSolution)}<br/>
          ({score(FIRST_PROBLEM_FREQUENCIES, firstSolution).toFixed(3)} bit[s] per symbol)
        </output>
      </label><br/><br/>
      <label>
        Frequencies: {formatMap(SECOND_PROBLEM_FREQUENCIES)}<br/>
        Chosen Encoding:
        <output>
          {formatMap(secondSolution)}<br/>
          ({score(SECOND_PROBLEM_FREQUENCIES, secondSolution).toFixed(3)} bit[s] per symbol)
        </output>
      </label>
    </div>
  );
}
